varying lowp vec4 v_color;
varying lowp vec2 v_texCoord;
uniform sampler2D u_texture;
uniform sampler2D u_texture1;   // noise
uniform vec4 u_params;
uniform vec4 u_extra_params; // x = timer

void main()
{
	float distortionFactor = u_params.z;
	float riseFactor = u_params.w;
	vec2 distortionMapCoordinate = v_texCoord.st;

	// We use the time value to scroll our distortion texture upwards
	// Since we enabled texture repeating, OpenGL takes care of
	// coordinates that lie outside of [0, 1] by discarding
	// the integer part and keeping the fractional part
	// Basically performing a "floating point modulo 1"
	// 1.1 = 0.1, 2.4 = 0.4, 10.3 = 0.3 etc.
	distortionMapCoordinate.t -= u_extra_params.x * riseFactor;

	vec4 distortionMapValue = texture2D(u_texture1, distortionMapCoordinate);

	// The values are normalized by OpenGL to lie in the range [0, 1]
	// We want negative offsets too, so we subtract 0.5 and multiply by 2
	// We end up with values in the range [-1, 1]
	vec2 distortionPositionOffset = distortionMapValue.xy;
	distortionPositionOffset = distortionPositionOffset * 2.0 - 1.0;

	// The factor scales the offset and thus controls the severity
	distortionPositionOffset *= distortionFactor;

	// Since we all know that hot air rises and cools,
	// the effect loses its severity the higher up we get
	// We use the t (a.k.a. y) texture coordinate of the original texture
	// to tell us how "high up" we are and damp accordingly
	// Remember, OpenGL 0 is at the bottom
	distortionPositionOffset *= (1. - v_texCoord.t);
	vec2 distortedTextureCoordinate = v_texCoord.st;// + distortionPositionOffset;
	float dist = length(v_texCoord - vec2(u_params.x,  u_params.y));
	dist = clamp(dist, 0.0, 0.1);
	distortedTextureCoordinate.x += mix(distortionPositionOffset.x, 0.0, dist / 0.1);
	distortedTextureCoordinate.y += mix(distortionPositionOffset.y, 0.0, dist / 0.1);
	
	gl_FragColor = v_color * texture2D(u_texture,  distortedTextureCoordinate);
}